@@ -40,6 +40,11 @@ import sys
 # 2. crayons is a wrapper around colorama, not in debian
 import termcolor
 
+# we use dateparser to handle -d but also fallback on parsedatetime or
+# "now" if both are missing
+#
+# another option could be https://pypi.org/project/arrow/ which does
+# date parsing, `relativedate`, `pytz`, and more...
 try:
     import dateparser
 except ImportError:  # pragma: no cover
@@ -64,6 +69,8 @@ import yaml
 # tabulate is similar: https://pypi.python.org/pypi/tabulate
 # texttable as well: https://github.com/foutaise/texttable/
 # terminaltables is the full thing: https://robpol86.github.io/terminaltables/
+# "rich" has more features, but awkward interface: https://rich.readthedocs.io/en/stable/reference/table.html
+# ie. you need to add rows one at a time, not worth it
 
 # originally, i was just centering thing with the .format()
 # handler. this was working okay except that it was too wide because i
@@ -116,11 +123,14 @@ class NegateAction(argparse.Action):
         setattr(ns, self.dest, not option.startswith(self.negative))
 
 
-class ConfigAction(argparse.Action):
+class ConfigAction(argparse._StoreAction):
     """add configuration file to current defaults.
 
     a *list* of default config files can be specified and will be
     parsed when added by ConfigArgumentParser.
+
+    it was reported this might not work well with subparsers, patches
+    to fix that are welcome.
     """
 
     def __init__(self, *args, **kwargs):
@@ -132,11 +142,10 @@ class ConfigAction(argparse.Action):
         """change defaults for the namespace, still allows overriding
         from commandline options"""
         for path in values:
-            config = self.parse_config(path)
-            if config:
-                parser.set_defaults(**config)
+            parser.set_defaults(**self.parse_config(path))
+        super().__call__(parser, ns, values, option)
 
-    def parse_config(self, path):  # pragma: no cover
+    def parse_config(self, path):
         """abstract implementation of config file parsing, should be overriden in subclasses"""
         raise NotImplementedError()
 
@@ -148,7 +157,7 @@ class YamlConfigAction(ConfigAction):
         try:
             with open(os.path.expanduser(path), "r") as handle:
                 logging.debug("parsing path %s as YAML" % path)
-                return yaml.safe_load(handle)
+                return yaml.safe_load(handle) or {}
         except (FileNotFoundError, yaml.parser.ParserError) as e:
             raise argparse.ArgumentError(self, e)
 
@@ -165,26 +174,107 @@ class ConfigArgumentParser(argparse.ArgumentParser):
 
             self.add_argument('--config', action=YamlConfigAction, default=self.default_config())
 
+    >>> from tempfile import NamedTemporaryFile
+    >>> c = NamedTemporaryFile()
+    >>> c.write(b"foo: delayed\\n")
+    13
+    >>> c.flush()
+    >>> parser = ConfigArgumentParser()
+    >>> a = parser.add_argument('--foo', default='bar')
+    >>> a = parser.add_argument('--config', action=YamlConfigAction, default=[c.name])
+    >>> args = parser.parse_args([])
+    >>> args.config == [c.name]
+    True
+    >>> args.foo
+    'delayed'
+
+    This is the same test, but with `--config` called earlier, which
+    should still work (but doesn't, because the subsequent action
+    changes the default).
+
+    >>> from tempfile import NamedTemporaryFile
+    >>> c = NamedTemporaryFile()
+    >>> c.write(b"foo: quux\\n")
+    10
+    >>> c.flush()
+    >>> parser = ConfigArgumentParser()
+    >>> a = parser.add_argument('--config', action=YamlConfigAction, default=[c.name])
+    >>> a = parser.add_argument('--foo', default='bar')
+    >>> args = parser.parse_args([])
+    >>> args.config == [c.name]
+    True
+    >>> args.foo
+    'quux'
+    >>> args = parser.parse_args(['--foo', 'baz'])
+    >>> args.foo
+    'baz'
+    >>> parser = ConfigArgumentParser()
+    >>> a = parser.add_argument('--config', action=YamlConfigAction, default=[c.name])
+    >>> a = parser.add_argument('--foo', default='bar')
+    >>> args = parser.parse_args(['--config', '/dev/null'])
+    >>> args.foo
+    'bar'
+    >>> args = parser.parse_args(['--config', '/dev/null', '--foo', 'baz'])
+    >>> args.foo
+    'baz'
+    >>> c.close()
+
     """
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # a list of actions to fire with their defaults if not fired
+        # during parsing
+        self._delayed_config_action = []
+
     def _add_action(self, action):
         # this overrides the add_argument() routine, which is where
-        # actions get registered. it is done so we can properly load
-        # the default config file before the action actually gets
-        # fired. Ideally, we'd load the default config only if the
-        # action *never* gets fired (but still setting defaults for
-        # the namespace) but argparse doesn't give us that opportunity
-        # (and even if it would, it wouldn't retroactively change the
-        # Namespace object in parse_args() so it wouldn't work).
+        # actions get registered in the argparse module.
+        #
+        # we do this so we can properly load the default config file
+        # before the the other arguments get set.
+        #
+        # now, of course, we do not fire the action here directly
+        # because that would make it impossible to *not* load the
+        # default action. so instead we register this as a
+        # "_delayed_config_action" which gets fired in `parse_args()`
+        # instead
         action = super()._add_action(action)
         if isinstance(action, ConfigAction) and action.default is not None:
-            # fire the action, later calls can override defaults
+            self._delayed_config_action.append(action)
+
+    def parse_args(self, args=None, namespace=None):
+        # we do a first failsafe pass on the commandline to find out
+        # if we have any "config" parameters specified, in which case
+        # we must *not* load the default config file
+        ns, _ = self.parse_known_args(args, namespace)
+
+        # load the default configuration file, if relevant
+        #
+        # this will parse the specified config files and load the
+        # values as defaults *before* the rest of the commandline gets
+        # parsed
+        #
+        # we do this instead of just loading the config file in the
+        # namespace precisely to make it possible to override the
+        # configuration file settings on the commandline
+        for action in self._delayed_config_action:
+            if action.dest in ns and action.default != getattr(ns, action.dest):
+                # do not load config default if specified on the commandline
+                logging.debug("not loading delayed action because of config override")
+                # action is already loaded, no need to parse it again
+                continue
             try:
-                action(self, None, action.default, None)
+                action(self, ns, action.default, None)
                 logging.debug("loaded config file: %s" % action.default)
             except argparse.ArgumentError as e:
                 # ignore errors from missing default
                 logging.debug("default config file %s error: %s" % (action.default, e))
+        # this will actually load the relevant config file when found
+        # on the commandline
+        #
+        # note that this will load the config file a second time
+        return super().parse_args(args, namespace)
 
     def default_config(self):
         """handy shortcut to detect commonly used config paths"""
@@ -228,12 +318,15 @@ class UndertimeArgumentParser(ConfigArgumentParser):
         super().__init__(
             description="pick a meeting time", epilog=__doc__, *args, **kwargs
         )
-        self.add_argument(
-            "timezones",
-            nargs="*",
+        group = self.add_argument_group("time and date options")
+        group.add_argument(
+            "-t",
+            "--timezones",
+            nargs="+",
+            default=[],
             help="time zones to show [default: current time zone]",
         )
-        self.add_argument(
+        group.add_argument(
             "-s",
             "--start",
             default=9,
@@ -241,7 +334,7 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             metavar="HOUR",
             help="start of working day, in hours [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "-e",
             "--end",
             default=17,
@@ -249,65 +342,92 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             metavar="HOUR",
             help="end of working day, in hours [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "-d",
             "--date",
             default=None,
+            help=argparse.SUPPRESS,
+        )
+        group.add_argument(
+            "datelist",
+            default=None,
+            nargs="*",
             metavar="WHEN",
             help='target date for the meeting, for example "in two weeks" [default: now]',
         )
-        self.add_argument(
+        group = self.add_argument_group("display options")
+        group.add_argument(
             "--no-colors",
             "--colors",
             action=NegateAction,
             dest="colors",
-            default=sys.stdout.isatty() and "NO_COLOR" not in os.environ,
+            default=sys.stdout.isatty()
+            and "NO_COLOR" not in os.environ
+            and "UNDERTIME_NO_COLOR" not in os.environ,
             help="show colors [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "--no-default-zone",
             "--default-zone",
             action=NegateAction,
             dest="default_zone",
             help="show current time zone first [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "--no-unique",
             "--unique",
             action=NegateAction,
             dest="unique",
             help="deduplicate time zone offsets [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "--no-overlap",
             "--overlap",
             action=NegateAction,
             dest="overlap_show",
             help="show zones overlap [default: %(default)s]",
         )
-        self.add_argument(
+        group.add_argument(
             "--overlap-min",
             default=0,
             type=int,
             metavar="N",
             help="minimum overlap between zones [default: %(default)s]",
         )
-        self.add_argument(
-            "--abbreviate",
-            "--no-abbreviate",
-            dest="abbreviate",
+        group.add_argument(
+            "--truncate",
+            "--no-truncate",
+            dest="truncate",
             action=NegateAction,
             help="short column headers [default: %(default)s]",
         )
-        self.add_argument(
+        # backwards-compatibility for config file and scripts
+        group.add_argument(
+            "--abbreviate",
+            "--no-abbreviate",
+            dest="truncate",
+            action=NegateAction,
+            help=argparse.SUPPRESS,
+        )
+        group.add_argument(
             "-f",
             "--format",
             default="fancy_grid_nogap",
             choices=tabulate.tabulate_formats + ["fancy_grid_nogap"],
             help="output format (%(default)s)",
         )
+        group.add_argument(
+            "--no-table",
+            "--table",
+            dest="show_table",
+            action=NegateAction,
+            help="hide/show the actual table [default: %(default)s]",
+        )
         self.add_argument(
-            "--config", action=YamlConfigAction, default=self.default_config()
+            "--config",
+            action=YamlConfigAction,
+            default=self.default_config(),
+            help="configuration file [default: %(default)s]",
         )
         self.add_argument(
             "-v",
@@ -322,22 +442,37 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             const="DEBUG",
             help="enable debugging messages",
         )
-        self.add_argument(
+        group = self.add_argument_group("commands")
+        group.add_argument(
             "-l",
             "--list-zones",
             action="store_true",
             help="show valid time zones and exit",
         )
-        self.add_argument(
+        group.add_argument(
             "--selftest", action="store_true", help="run test suite and exit"
         )
-        self.add_argument(
+        group.add_argument(
             "-V",
             "--version",
             action=ImportlibVersionAction,
             help="print version number and exit",
         )
 
+    def parse_args(self, args=None, namespace=None):
+        """override argument parser to properly interpret dates
+
+        This is mostly a remnant of when this was a option like
+        --date. Now we take all remaining commandline arguments and
+        treat them as a space-separated date.
+        """
+        ns = super().parse_args(args, namespace)
+        if ns.datelist and ns.date:
+            self.error("date specified as an argument an option, aborting")
+        if ns.datelist and not ns.date:
+            ns.date = " ".join(ns.datelist)
+        return ns
+
 
 class OffsetZone(pytz._FixedOffset):
     """Parse an offset from a human-readable string
@@ -500,7 +635,7 @@ def main(args):
     #doctest:ELLIPSIS
     >>> argv = ['--config', '/dev/null', '--no-default', '--no-unique']
     >>> date = ['--date', "2020-03-08 12:00:00-04:00"]
-    >>> zones = ['EST', 'US/Eastern', 'UTC']
+    >>> zones = ['--timezone', 'EST', 'US/Eastern', 'UTC']
     >>> args = UndertimeArgumentParser().parse_args(argv + date + zones)
     >>> main(args)
     ╔════════╤══════════════╤════════╤═══════════╗
@@ -538,7 +673,7 @@ def main(args):
     ...
     Local time requested: 2020-11-01 16:00:00+00:00
     Equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
-    >>> extra = [ '--no-overlap', '--abbreviate' ]
+    >>> extra = [ '--no-overlap', '--truncate' ]
     >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
     >>> main(args)
     ╔════════╤═══════════╤════════╗
@@ -546,7 +681,7 @@ def main(args):
     ╠════════╪═══════════╪════════╣
     ║ 19:00  │   20:00   │ 00:00  ║
     ...
-    >>> extra = [ '--overlap', '--abbreviate' ]
+    >>> extra = [ '--overlap', '--truncate' ]
     >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
     >>> main(args)
     ╔════════╤═══════════╤════════╤═════╗
@@ -556,7 +691,14 @@ def main(args):
     ...
     >>> logging.getLogger('').setLevel('WARNING')
     >>> argv = ['--config', '/dev/null', '--no-default']
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + [ 'CEST' ])
+    >>> args = UndertimeArgumentParser().parse_args(argv + date + [ '--timezones', 'CEST' ])
+    >>> main(args)
+    Traceback (most recent call last):
+        ...
+    SystemExit: 1
+    >>> logging.getLogger('').setLevel('WARNING')
+    >>> argv = ['--config', '/dev/null', '--no-default']
+    >>> args = UndertimeArgumentParser().parse_args(argv)
     >>> main(args)
     Traceback (most recent call last):
         ...
@@ -573,6 +715,7 @@ def main(args):
     Traceback (most recent call last):
         ...
     SystemExit: 0
+    >>> args = UndertimeArgumentParser().parse_args(argv + ["2020-03-08", "12:00:00-04:00"])
     >>>
     """
     if args.list_zones:
@@ -583,17 +726,14 @@ def main(args):
         global fmt_time
         fmt_time = fmt_time_ascii
 
-    # get the current UTC time
-    now_utc = datetime.datetime.now(datetime.timezone.utc)
-    # ... to extract the local system's time zone
-    # https://stackoverflow.com/a/39079819/1174784
-    now_zone = now_utc.astimezone().tzinfo
+    # find the local timezone
+    now_zone = get_localzone()
     # make an educated guess at what the user meant by passing that time zone to parse_date
     then_local = parse_date(args.date, now_zone).replace(second=0, microsecond=0)
     # convert that time to UTC again
     then_utc = then_local.astimezone(datetime.timezone.utc)
     # and guess what time zone that was *then*
-    then_zone = then_utc.astimezone().tzinfo
+    then_zone = get_localzone(then_utc)
 
     timezones = []
     if args.default_zone:
@@ -606,32 +746,33 @@ def main(args):
         logging.error("No valid time zone found.")
         sys.exit(1)
 
-    rows = compute_table(
-        then_local,
-        timezones,
-        args.start,
-        args.end,
-        overlap_min=args.overlap_min,
-        overlap_show=args.overlap_show,
-        abbreviate=args.abbreviate,
-    )
-    # reproduce the terminaltables DoubleTable output in tabulate:
-    # https://github.com/cmck/python-tabulate/issues/1
-    if args.format == "fancy_grid_nogap":
-        args.format = tabulate.TableFormat(
-            lineabove=tabulate.Line("╔", "═", "╤", "╗"),
-            linebelowheader=tabulate.Line("╠", "═", "╪", "╣"),
-            linebetweenrows=None,
-            linebelow=tabulate.Line("╚", "═", "╧", "╝"),
-            headerrow=tabulate.DataRow("║", "│", "║"),
-            datarow=tabulate.DataRow("║", "│", "║"),
-            padding=1,
-            with_header_hide=None,
+    if args.show_table:
+        rows = compute_table(
+            then_local,
+            timezones,
+            args.start,
+            args.end,
+            overlap_min=args.overlap_min,
+            overlap_show=args.overlap_show,
+            truncate=args.truncate,
         )
-    table = tabulate.tabulate(
-        rows, tablefmt=args.format, headers="firstrow", stralign="center"
-    )
-    print(table)
+        # reproduce the terminaltables DoubleTable output in tabulate:
+        # https://github.com/cmck/python-tabulate/issues/1
+        if args.format == "fancy_grid_nogap":
+            args.format = tabulate.TableFormat(
+                lineabove=tabulate.Line("╔", "═", "╤", "╗"),
+                linebelowheader=tabulate.Line("╠", "═", "╪", "╣"),
+                linebetweenrows=None,
+                linebelow=tabulate.Line("╚", "═", "╧", "╝"),
+                headerrow=tabulate.DataRow("║", "│", "║"),
+                datarow=tabulate.DataRow("║", "│", "║"),
+                padding=1,
+                with_header_hide=None,
+            )
+        table = tabulate.tabulate(
+            rows, tablefmt=args.format, headers="firstrow", stralign="center"
+        )
+        print(table)
     times = []
     for zone in timezones:
         times.append(
@@ -642,6 +783,48 @@ def main(args):
     print("Equivalent to: " + ", ".join(times))
 
 
+def get_localzone(now_utc=None):
+    """find the time zone on the local system
+
+    This was mostly cargo-culted from:
+
+    https://stackoverflow.com/a/39079819/1174784
+
+    ... but it also happens to work. To avoid such cargo-culting, we'd
+    need a entire module and I'm unwilling to add a dependency just to
+    remove this single function:
+
+    https://github.com/regebro/tzlocal
+
+    This is still a "tricky use" of datetime's code (one would say a
+    horrible hack), which, arguably, should really just be part of the
+    datetime module itself.
+
+    This unit test will succeed only if TZ=UTC is set in the
+    environment (which it is, in tox):
+
+    >>> n = datetime.datetime(2019, 5, 18, 15, 17, tzinfo=datetime.timezone.utc)
+    >>> str(get_localzone(n))
+    'UTC'
+
+    It's also kind of dumb because it somewhat proves nothing: it
+    doesn't show that it can find other timezones. But doing so would
+    require monkeypatching that environment, and I'm not quite ready
+    to mess with that yet. And besides, this comment is now about a
+    hundred times longer than the actual frigging code so I better
+    stop writing.
+    """
+    if now_utc is None:
+        # first we get a timestamp with the the UTC timezone
+        now_utc = datetime.datetime.now(datetime.timezone.utc)
+    # then, mysteriously, getting the timezone info back from *that*
+    # object gives us the *local* timezone. go figure. it's probably
+    # because we do not pass an argument to `astimezone` so it does
+    # our work of finding the right timezone, but really, that's just
+    # a guess at this point.
+    return now_utc.astimezone().tzinfo
+
+
 def guess_zone(zone):
     """
     guess a zone from a string, based on pytz
@@ -714,7 +897,7 @@ def uniq_zones(timezones, now):
 
 
 def compute_table(
-    now_local, timezones, start, end, overlap_min=0, overlap_show=True, abbreviate=False
+    now_local, timezones, start, end, overlap_min=0, overlap_show=True, truncate=False
 ):
     """
     >>> local_zone = datetime.timezone(datetime.timedelta(days=-1, seconds=72000), 'EDT')
@@ -752,7 +935,7 @@ def compute_table(
     # the first line is the list of time zones
     line = []
     for t in timezones:
-        if abbreviate:
+        if truncate:
             try:
                 prefix, suffix = str(t).split("/", 1)
             except ValueError:
@@ -761,7 +944,7 @@ def compute_table(
         else:
             line.append(str(t))
     if overlap_show:
-        if abbreviate:
+        if truncate:
             line.append("n")
         else:
             line.append("overlap")
